<?php
/**
 * mainFrame
 *
 * Display main frame for themes using window (and optionaly icons)
 * Set JS scripts for icons and windows handling
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

/**
 * @desc Add an icon on main frame
 * @param string $id : icon Id
 * @param string $img : icon's image path
 * @param string $text : icon caption
 * @param string $script : javascript to be executed on click
 * @param string $defaultPosition (optional) : css description of default icon's position
 * @param string $extraHTML (optional) : top div extra HTML
 * @param string $extraStyle (optional) : top div extra style
 * @return string: js array with:
 * 			id(0), left(1), top(2), img url(3), icon caption(4), onclick script(5), extraHTML(5), extraStyle(6)
*/
function addIcon($id, $img, $text, $script, $defaultPosition=false, $extraHTML=false, $extraStyle=false){
	// Compute position
	$top=-500; $left=-500;
	if(cfGGetVar('activeItems')){
		foreach (cfGGetVar('activeItems') as $value2)
			if($value2['type']=='icon' && $value2['name']=='resIcon_'.$id && $value2['x']!==null && $value2['y']!==null) {
				$top =$value2['y'];
				$left=$value2['x'];
			}
	}
	$ico='["'.$id.'",'.$left.','.$top.',"'.addslashes($img).'","'.addslashes(str_replace('>','&gt;',str_replace('<','&lt;',cfUTF8Encode($text)))).'","'.addslashes($script).'","'.addslashes($extraHTML).'","'.addslashes($extraStyle).'"]';
	return $ico;
}

/**
 * @desc Insert HTML base code of icons on main frame
 *
 * @param array $resources from which icons are generated
 */
function addIcons($resources){
?>
<script type="text/javascript">
var iconsList;
(function(){
	var i,o='',u,it=[],f;
	iconsList=[<?php
	$nb=0;
	foreach($resources as $id=>$value)	echo (($nb++)?',':'').addIcon($value['id'],$value['resourceIcon'],$value['name'],$value['resourceJsLink']);
	?>];
for(i=0;i<iconsList.length;i++) {
	u=iconsList[i];
	o+='<div name="resIcon2" id="resIcon_'+u[0]+'" class="iconDiv" style="left:'+u[1]+'px;top:'+u[2]+'px" onclick="if(!dragCancelOnclick && !menuShown && !W.dd.dragClick) {'+u[5]+'};dragCancelOnclick=false;if(W.event) stopPropagation(event)">';
	o+='<a class="icon" href="javascript:void(0)" title="'+((u[4]).replace(new RegExp('"', 'g'),'\\"'))+'"<?php if(cfGetBrowser()!=='ie') echo ' style="visibility:hidden"';?>>';
	o+='<img src="'+u[3]+'" alt="" id="resIcon_'+u[0]+'_img" onload="fade(this.parentNode)">';
	o+='<span id="iconCaption_'+u[0]+'_cont" class="iconC"></span>';
	o+='<span id="iconCaption_'+u[0]+'" class="iconC hidden">'+u[4]+'</span></a></div>';
}
D.write(o);
})();
</script>
<?php
}

/**
 * @desc update active items (windows and background icons) on server from POST data sent from client's mainFrame
 *
 */
function updateActiveItems(){
	$nb=0;
	// get icon data, discard win data
	$opWin=array();
	if(cfGGetVar('activeItems')) foreach (cfGGetVar('activeItems') as $key=>$value) if($value['type']=='icon') $opWin[$key]=$value;


	/**
	 * POST Format :
	 * winUpdate => true
	 * winActive => HTML id of active window
	 *
	 * for each windows and icons : X : incremental numeric value
	 *
	 * icons and windows:
	 * name_X => resIcon_6a7a427b01 (resource id)
	 * type_X => "icon" or "win"
	 * x_X => icon or window x position
	 * y_X => icon or window y position
	 *
	 * windows only:
	 * src_X => windows content's URL
	 * icon_X => windows icon's HTML code (<img src=...>)
	 * reloadable_X => true / false
	 * resizable_X => true / false
	 * closeable_X => true / false
	 * winState_X => normal / max / minMax / minNormal
	 */
	while (isset($_POST['name_'.$nb])) {
		// Background icons :
		if($_POST['type_'.$nb]=='icon'){
			$id=substr($_POST['name_'.$nb],strlen('resIcon_'));
		}
		else $id=$nb;
		if(!isset($opWin[$id])) $opWin[$id]=array();
		// Common icons / windows parameters
		$opWin[$id]['name']=$_POST['name_'.$nb];
		$opWin[$id]['type']=$_POST['type_'.$nb];
		$opWin[$id]['x']=$_POST['x_'.$nb];
		$opWin[$id]['y']=$_POST['y_'.$nb];

		// Windows specific parameters
		if($opWin[$id]['type']=='win'){
			$opWin[$id]['src']=$_POST['src_'.$nb];
			if(isset($_POST['icon_'.$nb])) $opWin[$id]['icon']=stripcslashes($_POST['icon_'.$nb]);
			$opWin[$id]['capt']=(isset($_POST['capt_'.$nb])) ? html_entity_decode($_POST['capt_'.$nb]): '';
			$opWin[$id]['reloadable']=$_POST['reloadable_'.$nb];
			$opWin[$id]['resizable']=$_POST['resizable_'.$nb];
			$opWin[$id]['closeable']=$_POST['closeable_'.$nb];
			$opWin[$id]['noDrag']=(isset($_POST['noDrag_'.$nb])) ? 'true' : 'false';
			$opWin[$id]['winState']=$_POST['winState_'.$nb];
			$opWin[$id]['w']=$_POST['w_'.$nb];
			$opWin[$id]['h']=$_POST['h_'.$nb];
			if(isset($_POST['winActive']) && $_POST['winActive']==$opWin[$id]['name']) $opWin[$id]['active']=true;
		}
		$nb++;
	}
	cfGSetVar('activeItems',$opWin);
	cfAsyncHeader();
	die(cfAsyncFooter());
}

/**
 * Get background Wallpaper and wallpaper position (centered / tiled / streched)
 *
 * @param bool $theme: true to return theme's wallpaper
 * @return array (path to wallpaper, wallpaper type)
 */
function getBackgroundWallpaper($theme=false){
	// User defined wallpaper
	if((!$theme) && WEnv::user()->getVar('accountType')=='singleUser' && ($wp=WEnv::user()->getVar('wallpaper')) && cfBGetVar('name')!='iPhone'){
		if($wp=='no') return array('no',false); // User requested no wallpaper
		// User did not requested theme's default wallpaper
		if($wp!=='theme')
			return array('/gfx/wallpapers/'.$wp,((WEnv::user()->getVar('wallpaperPosition'))?WEnv::user()->getVar('wallpaperPosition'):'centered'));
	}

	// List wallpapers located into theme directory
	$wps=glob(cfAppThemeDir(false).'/{wallpaperBoth,themeBoth,wallpaper,theme}-{centered,stretched,tiled}.{jpg,gif,png}',GLOB_BRACE);
	
	if(count($wps)){
		foreach ($wps as $key=>$wp) {
			list($type,$position)=explode('-',cfFileWithoutExtension(basename($wp)));
			$wps[$key]=array('cfn'=>$wp,'type'=>$type,'position'=>$position);
		}
		
		// Look for user-set login wallpaper
		if(!$theme) foreach ($wps as $key=>$wp) if($wp['type']=='wallpaper') return array(cfAppThemeDir(true).'/'.basename($wp['cfn']),$wp['position']);

		// Look for user-set login+desktop wallpaper
		if(!$theme) foreach ($wps as $key=>$wp) if($wp['type']=='wallpaperBoth') return array(cfAppThemeDir(true).'/'.basename($wp['cfn']),$wp['position']);
		
		// Look for theme login wallpaper
		foreach ($wps as $key=>$wp) if($wp['type']=='theme') return array(cfAppThemeDir(true).'/'.basename($wp['cfn']),$wp['position']);

		// Look for theme login+desktop wallpaper
		foreach ($wps as $key=>$wp) if($wp['type']=='themeBoth') return array(cfAppThemeDir(true).'/'.basename($wp['cfn']),$wp['position']);
	}

	return array(false,false);
}


/**
 * Mobile browsers
 */
if(cfIsMobile('Wii')){
	require(W_THEMES_DIR.'mobile/mainFrame.php');
	exit;
}

/**
 * Get wallpaper
 */
require(INCLUDE_DIR.'outputFunctions.php');
require_once(INCLUDE_DIR.'framesFunctions.php');

list($wallpaper, $wallpaperPosition )=getBackgroundWallpaper();
list($tWallpaper,$tWallpaperPosition)=getBackgroundWallpaper('theme');
//cfDbg($wallpaper.$wallpaperPosition.' <-> '.$tWallpaper.$tWallpaperPosition);
/*
 ***************************************************************************************************************************
 * Process POST commands
 ***************************************************************************************************************************
 */

/*
 * Window / icon position update command
 */
if(isset($_POST['winUpdate'])) updateActiveItems();

/*
 * Sticky notes position & edition commands
 */
if(isset($_POST['sNotesAction'])) {
	require_once(INCLUDE_DIR.'stickyNotes.php');
	sNotesUpdate();
}

/*
 * Insert HEAD
 */
cfInsertHEAD(false);

// Theme specific head data
$mainFrameThemeData=cfTGetVar('mainFrame');
if($mainFrameThemeData['extraHeader']) echo $mainFrameThemeData['extraHeader']."\n";
echo cfScriptLink('win.js');
?>
<script language="javascript" type="text/javascript">
var winCaptionReload="<?php echo cfCaptionJS('winReload');?>",
winCaptionRestore="<?php echo cfCaptionJS('winRestore');?>",
winCaptionMaximize="<?php echo cfCaptionJS('winMaximize');?>",
winCaptionMinimize="<?php echo cfCaptionJS('winMinimize');?>",
winCaptionTNW="<?php echo cfCaptionJS('musicPopupPlayer');?>",
winCaptionClose="<?php echo cfCaptionJS('winClose');?>",
winCaptionConfig="<?php echo cfCaptionJS('mainConfigButton');?>",
winCaptionLogout="<?php echo cfCaptionJS('genLogout');?>",
winIconConfig="<?php echo outIcon('config3');?>",
winIconLogout="<?php echo outIcon('logout');?>",
windowsAnimation=<?php echo (int)cfHGetVar('windowsAnimation');?>,
winLimitArea=1, isMainFrame=1,
winPadding={'left':0,'top':0,'right':0,'bottom':0},
wallpaper="<?php echo $wallpaper;?>", wallpaperPosition="<?php echo $wallpaperPosition;?>",
tWallpaper="<?php echo $tWallpaper;?>",
tWallpaperPosition="<?php echo $tWallpaperPosition;?>",
desktopColor="<?php if(WEnv::user()->getVar('desktopColor')) echo WEnv::user()->getVar('desktopColor'); else echo '-1'; ?>",
wpImageStretched=new Image(),
scriptLocation="<?php echo $_SERVER['PHP_SELF'];?>",
winConfigBt=function(rId,opt){return '<?php echo (str_replace('<',"<'+'",outButton(cfCaption('genConfigure'),"javascript:void(winIF(\'Win_'+rId+'\').document.location=\\''+opt['remoteConfigFile']+'\\')",outIcon('gears'))));?>';};
<?php
// Menu item(s) padding
if(cfTGetVar('framelessDesktop')){
	$menuPosition=cfTGetVar('menuFrame');
	echo 'winPadding.'.$menuPosition['position'].'='.$menuPosition['size'].";\n";
}

// if frameless display, set ping
if(!cfTGetVar('frames')) echo 'var pingInt=setInterval("ping()",'.(max(cfGGetVar('clientPingInterval'),1)*1000).');';

if(cfTGetVar('winEmptyHTML')) echo 'winEmptyHTML=\''.str_replace('<','\'+\'<\'+\'',cfTGetVar('winEmptyHTML')).'\';'; else echo 'winEmptyHTML=\'<\'+\'body style="background:white"><\'+\'/body>\';'; // To get a Valid HTML 4.01 Transitional page...
echo "\n";

// Set opened windows properties array
echo "var activeItems=[";
if(is_array(cfGGetVar('activeItems'))){
	$nb=0;
	foreach (cfGGetVar('activeItems') as $key=>$value){
		if($value['type']=='win'){
		echo ($nb==0)?'{':',{';
			echo '"name":"'.$value['name'].'"';
			echo ',"type":"'.$value['type'].'"';
			echo ',"x":'.$value['x'].',"y":'.$value['y'];
			if(isset($value['capt'])) echo ',"capt":"'.htmlspecialchars($value['capt']).'"';
			echo ',"src":"'.str_replace('"','\\"',$value['src']).'"';
			if(isset($value['icon'])) echo ',"icon":"'.addslashes($value['icon']).'"';
			echo ',"reloadable":'.$value['reloadable'];
			echo ',"resizable":'.$value['resizable'];
			echo ',"closeable":'.$value['closeable'];
			echo ',"noDrag":'.$value['noDrag'];
			echo ',"winState":"'.$value['winState'].'"';
			echo ',"w":'.$value['w'].',"h":'.$value['h'];
			if(isset($value['active'])) echo ',"active":true';
			echo '}';
			$nb++;
		}
	}
}
echo '];';


// Restore windows positions and open default or requested resource if appliable
?>
function bgSetWindows(){
	winRestorePosition(activeItems);
	<?php
	// If client requested a startup resource in login URL GET "res" parameter
	if(WEnv::user()->getVar('displayResourceAtLogin') && (!cfIsMobile('iPhone')||count($_SESSION['res'])==1)){
		echo WEnv::resLinked(WEnv::user()->getVar('displayResourceAtLogin'))->getVar('resourceJsLink').";\n";
		WEnv::user()->setVar('displayResourceAtLogin',false);
	}
	// If account has a single resource, open it
	elseif(!WEnv::config()->getVar('firstPageDisplayed') && (count($_SESSION['res'])==1)) echo $_SESSION['res'][0]['resourceJsLink'].";\n";

	WEnv::config()->setVar('firstPageDisplayed',true);

	// Hide menu mask if present
	?>
	if(dd.elements['menuMask']) dd.elements['menuMask'].hide();
}
</script>
<?php

/**
 * Insert scripts
 */
echo cfScriptLink('mainFrame.js',true);

?>
</head>

<?php

/* Body */
echo '<body class="mainFrameBody" onload="bgBodyLoaded()" style="overflow:hidden';

// Set background color
if(substr(WEnv::user()->getVar('wallpaper'),0,1)=='-' && WEnv::user()->getVar('accountType')=='singleUser' && cfBGetVar('name')!='iPhone')
	echo ';background-color:#'.substr(WEnv::user()->getVar('wallpaper'),1,6);
elseif(WEnv::user()->getVar('desktopColor') && WEnv::user()->getVar('wallpaper')!='theme' && WEnv::user()->getVar('accountType')=='singleUser' && cfBGetVar('name')!='iPhone')
	echo ';background-color:'.WEnv::user()->getVar('desktopColor');

// If wallpaper, remove theme background image not to load it unnecessarily
if($wallpaper && $wallpaper!=='theme') echo ';background-image:none';

echo '" ';

// Theme extra HTML
if($mainFrameThemeData['bodyExtraHTML']) echo ';'.$mainFrameThemeData['bodyExtraHTML'];
echo ">\n";
// Drag'n drop scripts
echo cfScriptLink('wz_dragdrop.js');
?>
<script language="javascript" type="text/javascript">;
var disconnectionInProgress=0;
var disconnectionSent=0;
function logout(){
	if(disconnectionInProgress) return;
	disconnectionInProgress=true;
	if(top!=parent.self) document.logoutForm.logout.value="top";
	D.logoutForm.submit();
}
var pingInt = window.setInterval('ping()',pingInterval);
wl.blockBackspace();
</script>
<form name="logoutForm" action="/index.php" target="_parent" enctype="multipart/form-data" method="POST" style="display:none"><input name="logout" value="parent"></form>
<div id="rSPb" class="frame3" style="display:none; position:absolute;left:5px;bottom:5px;"><?php echo cfCaption('menuServerPb');?></div>
<?php

// Div covering all desktop
echo '<div id="wpDiv" style="overflow:hidden;height:100%;width:100%;position:absolute;z-index:0"></div>';

// Windows "dock" div
echo '<div id="winDock"><div id="winBorder" style="position:absolute;display:none;visibility:hidden"></div></div>';

/**
 * Theme background script
 */
if(file_exists(cfAppThemeDir(false).'/background.php')) require_once(W_THEMES_DIR.cfActualTheme().'/background.php');

// Register drag items
echo cfDragRegisterItems(false);

/**
 * Sticky Notes
 */
if(WEnv::user()->getVar('sNotes')>0 && cfBGetVar('name')!='iPhone'){
	require_once(INCLUDE_DIR.'stickyNotes.php');
	sNotesDisplay();
}

// Initialize windows
?>
<form id="reloadForm" style="display:none" method="POST" action="" target=""><input name="a"><input name="reloadPostData"></form>
<script language="javascript" type="text/javascript">
bgHTMLLoaded();
</script>
</body></html>